@@ -5,6 +5,7 @@ from typing import Any, Dict, List, Union, overload
 from airbyte_cdk.models import (
     ConfiguredAirbyteCatalog,
     ConfiguredAirbyteStream,
+    ConfiguredAirbyteStreamSerializer,
     SyncMode,
 )
 
@@ -41,7 +42,7 @@ class ConfiguredAirbyteStreamBuilder:
         return self
 
     def build(self) -> ConfiguredAirbyteStream:
-        return ConfiguredAirbyteStream(**self._stream)
+        return ConfiguredAirbyteStreamSerializer.load(self._stream)
 
 
 class CatalogBuilder:
